#!/usr/bin/php
<?php

require_once('notify-nilex.inc.php');
error_reporting(E_ERROR | E_PARSE);

if ( isset($argv[1]) && ($argv[1] == "-h" || $argv[1] == "--help")) {
	$usage = <<<USAGE
This script is used to generate tickets in Nilex using the soap interface made available
by 'NIS'. To set it up complete the steps below.

Nilex preparation
  Make sure the 'integration module' is insalled on the Nilex server. Contact Nilex for
  installation and configuration instructions

ITRS OP5 Monitor Configuration
Step 1:
  Make sure atleast version 3.3.5 of the monitor-notify package is installed by executing:
    rpm -q monitor-notify
Step 2:
  Configure customerName, groupID, userSign, prioID, classificationID, categoryID and alertName
  in /opt/monitor/op5/notify/notify-nilex.inc.php.
  Contact Nilex for information on how to get these values out of the NIS database
Step 3:
  Make sure you have two notification commands for Nilex notifications named host-notify-nilex
  and service-notify-nilex. If these commands do not exist you can copy/create them using the
  suggestions in: /op5/notify/misccommands-nilex.cf
Step 4:
  Setup a op5 contact that uses host-notify-nilex and service-notify-nilex as notification
  commands. The contact should have the adress to the TicketIntegrationService.asmx in the
  address1 field
  For example: http://192.168.1.142/nis/TicketIntegrationService.asmx
Step 5:
  Optionally modify the close_on_ok and close_on_ack vars in notify-nilex.inc.php to customize
  the behaviour

To test the communication from the command line issue for example:

php notify-nilex.php url=http://<nilex-server-address>/nis/TicketIntegrationService.asmx
NOTIFICATIONTYPE=PROBLEM HOSTNAME="test-host" HOSTOUTPUT="test output" HOSTSTATE="UP"

USAGE;
	echo $usage;
	exit (1);
}

/* Check that the php-soap package is installed */
if ( ! class_exists("SoapClient") ) {
	$message = <<<MESSAGE
Error: Class 'SoapClient' do not exist, this indicates that the php-soap
package is not installed. To install try executing:
  yum install php-soap
This should install the required package.
MESSAGE;
	echo $message;
	exit(1);
}

if ( $customerName === "" ) {
	print_log("Error: customerName (and possible other parameters) are not set correctly.\n " .
			  "Edit setting in notify-nilex.inc.php and try again.");
	exit(1);
}

/* If to few args we bail */
if( $argc < 2 ) {
	print_log("To few arguments.");
	echo "Execute notify-nilex.php --help for usage instructions\n";
	exit(1);
}

/* Build an array of all input args */
$input_args = array();

/* Set some defaults, empty string mean we get server side defaults */
/* This is needed since empty SERVICEDESC mean this is a host alert */
$input_args['SERVICESTATE'] = "";
$input_args['HOSTSTATE']    = "";
$input_args['SERVICEDESC']  = "";
$input_args['HOSTOUTPUT']   = "";

$input_args['LASTSERVICECHECK']       = 0;
$input_args['LASTSERVICESTATECHANGE'] = 0;
$input_args['LASTHOSTCHECK']          = 0;
$input_args['LASTHOSTSTATECHANGE']    = 0;



/* Parse input args and place in assosiative array */
print_debug_log("Parsing arguments.");
for($i=1; $i<$argc; $i++){
	$tmp = split("=", $argv[$i], 2);
	if(isset($tmp[1])) {
		$input_args[$tmp[0]] = $tmp[1];
	}
}

/* Verify that all args are provided */
check_req_args($input_args);

/* Create the client */
print_debug_log("Creating the soap client.");
for($i=0;$i<3;$i++){
	try {
		$client = new SoapClient($input_args['wsdl'], array('location' => $input_args['url']));
	} catch (Exception $e) {
		print_debug_log("Failed to create soap client. Caught error: $e->faultcode, $e->faultstring");
		sleep(5);
		continue;
	}
	break;
}

if(!$client) {
	print_log("Failed to create soap client. Caught error: $e->faultcode, $e->faultstring");
	exit(1);
}
print_debug_log("Client created successfully");


function check_req_args($arg_arr) {
	Global $input_args;
	print_debug_log("Checking for required arguments.");
	if(!isset($arg_arr['url']))
		exit(0);
	if(!isset($arg_arr['wsdl'])) {
		$input_args['wsdl'] = $arg_arr['url'] . "?WSDL";
	}
	if(!isset($input_args['NOTIFICATIONTYPE'])) {
		print_log("Missing NOTIFICATIONTYPE");
		exit(1);
	}
	if(!isset($input_args['HOSTNAME'])){
		print_log("Missing HOSTNAME");
		exit(1);
	}
}



$new_notification = array(
	"NotificationType" => $input_args['NOTIFICATIONTYPE'],
	"ServiceState" => $input_args['SERVICESTATE'],
	"HostState" => $input_args['HOSTSTATE'],
	"HostName" => $input_args['HOSTNAME'],
	"ServiceName" => $input_args['SERVICEDESC']
);

$close_op5_notification_request = array();

if($input_args['SERVICEDESC'] != ""){
	/* Service notification */
	$LastCheckDate     = date("Y-m-d", $input_args['LASTSERVICECHECK']) . "T" . date("H:i:s", $input_args['LASTSERVICECHECK']) . "Z";
	$RaiseTime         = date("Y-m-d", $input_args['LASTSERVICESTATECHANGE']) . "T" . date("H:i:s", $input_args['LASTSERVICESTATECHANGE']) . "Z";
	$RegTime           = date("Y-m-d", $input_args['LASTSERVICESTATECHANGE']) . "T" . date("H:i:s", $input_args['LASTSERVICESTATECHANGE']) . "Z";
	$OP5Id             = $input_args['HOSTNAME'] . ":" . $input_args['SERVICEDESC'];
	$AlertName         = $alertName;
	$AlertDescription  = $input_args['SERVICEOUTPUT'];

	$resolutionText    = "Service " . $input_args['SERVICEDESC'] . " on " . $input_args['HOSTNAME'] .  " is " . $input_args['SERVICESTATE'];
	$technicalSolution = $input_args['SERVICEOUTPUT'];
	$closeTime         = date("Y-m-d", $input_args['LASTSERVICESTATECHANGE']) . "T" . date("H:i:s", $input_args['LASTSERVICESTATECHANGE']) . "Z";

	$new_notification['LastCheckDate']    = $LastCheckDate;
	$new_notification['RaiseTime']        = $RaiseTime;
	$new_notification['RegTime']          = $RegTime;
	$new_notification['OP5Id']            = $OP5Id;
	$new_notification['AlertName']        = $AlertName;
	$new_notification['AlertDescription'] = $AlertDescription;

	$close_op5_notification_request['op5ID']             = $OP5Id;
	$close_op5_notification_request['resolutionText']    = $resolutionText;
	$close_op5_notification_request['technicalSolution'] = $technicalSolution;
	$close_op5_notification_request['closeTime']         = $closeTime;
} else {
	/* Host notification */
	$LastCheckDate     = date("Y-m-d", $input_args['LASTHOSTCHECK']) . "T" . date("H:i:s", $input_args['LASTHOSTCHECK']) . "Z";
	$RaiseTime         = date("Y-m-d", $input_args['LASTHOSTSTATECHANGE']) . "T" . date("H:i:s", $input_args['LASTHOSTSTATECHANGE']) . "Z";
	$RegTime           = date("Y-m-d", $input_args['LASTHOSTSTATECHANGE']) . "T" . date("H:i:s", $input_args['LASTHOSTSTATECHANGE']) . "Z";
	$OP5Id             = $input_args['HOSTNAME'];
	$AlertName         = $alertName;
	$AlertDescription  = $input_args['HOSTOUTPUT'];

	$resolutionText    = "Host " . $input_args['HOSTNAME'] .  " is " . $input_args['HOSTSTATE'];
	$technicalSolution = $input_args['HOSTOUTPUT'];
	$closeTime         = date("Y-m-d", $input_args['LASTHOSTSTATECHANGE']) . "T" . date("H:i:s", $input_args['LASTHOSTSTATECHANGE']) ;
	

	$new_notification['LastCheckDate']    = $LastCheckDate;
	$new_notification['RaiseTime']        = $RaiseTime;
	$new_notification['RegTime']          = $RegTime;
	$new_notification['OP5Id']            = $OP5Id;
	$new_notification['AlertName']        = $AlertName;
	$new_notification['AlertDescription'] = $AlertDescription;
	$new_notification['ServiceState']     = "UNKNOWN";

	$close_op5_notification_request['op5ID']             = $OP5Id;
	$close_op5_notification_request['resolutionText']    = $resolutionText;
	$close_op5_notification_request['technicalSolution'] = $technicalSolution;
	$close_op5_notification_request['closeTime']         = $closeTime;

}

/* Build the notification request (not sure we need to send though) */
$add_op5_notification_request = array(
	"notification" => $new_notification,
	"customerName" => $customerName,
	"groupID" => $groupID,
	"userSign" => $userSign,
	"prioID" => $prioID,
	"classificationID" => $classificationID,
	"categoryID" => $categoryID
);

// Add information about current timezone setting which is needed in Nilex post 8.86.
// We need to make this dynamic if it's supposed to work anywhere outside of Sweden
// but so far we can keep it hardcoded
if(isset($nilexVersion) && $nilexVersion >= 8.86) {
	$add_op5_notification_request['timeZoneSerialized'] = "W. Europe Standard Time;60;(UTC+01:00) Amsterdam, Berlin, Bern, Rome, Stockholm, Vienna;W. Europe Standard Time;W. Europe Daylight Time;[01:01:0001;12:31:9999;60;[0;02:00:00;3;5;0;];[0;03:00:00;10;5;0;];];";
	$close_op5_notification_request['timeZoneSerialized'] = "W. Europe Standard Time;60;(UTC+01:00) Amsterdam, Berlin, Bern, Rome, Stockholm, Vienna;W. Europe Standard Time;W. Europe Daylight Time;[01:01:0001;12:31:9999;60;[0;02:00:00;3;5;0;];[0;03:00:00;10;5;0;];];";
}

if($close_on_ok === TRUE && $input_args['NOTIFICATIONTYPE'] === "RECOVERY"){
	/* Host/service recovered so close ticket */
	print_debug_log("Closing Nilex ticket with id: '" . $close_op5_notification_request['op5ID']  . "' (Recovered)");
	print_debug_log($close_op5_notification_request);
	$res = $client->closeOP5Notification($close_op5_notification_request);
	exit(0);
}

if($close_on_ack === TRUE && $input_args['NOTIFICATIONTYPE'] === "ACKNOWLEDGEMENT"){
	/* Problem was acknowledged, so close ticket */
	print_debug_log("Closing Nilex ticket with id: '" . $close_op5_notification_request['op5ID'] . "' (Acknowledged)");
	print_debug_log($close_op5_notification_request);
	$res = $client->closeOP5Notification($close_op5_notification_request);
	exit(0);
}

/* Ticket not closed so add/modify it */
print_debug_log("Updating Nilex ticket with id: '" . $add_op5_notification_request['notification']['OP5Id'] . "'" );
$res = $client->addNewOP5Notification($add_op5_notification_request);
exit(0);




/* Helper functions */
function print_debug_log($str){
	Global $DEBUG;
	if(isset($DEBUG) && $DEBUG === true)
		print_log($str);
}
function print_log($str){
	GLOBAL $logfile;
	if(!is_string($str)) {
		$str = var_export($str, true);
	}
	file_put_contents($logfile, date("Ymd G:i:s") . " - $str\n", FILE_APPEND);
	print date("Ymd G:i:s") . " - $str\n";
}
