#!/usr/bin/perl -w
# 
# notify_dial.pl - Notification script for ITRS OP5 Monitor 
#                  that simply dials a number.
#
# License: GPL
# Copyright (c) 2005-2009 op5 AB
# Author: op5 dev team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Note: To allow sudo to be used in this script uncomment 'Defaults requiretty'
#       in /etc/sudoers
#

use strict;
use Device::Modem;
use Getopt::Long;
use File::Spec;

$SIG{'INT'} = 'exit_script';
$SIG{'HUP'} = 'exit_script';
$SIG{'QUIT'} = 'exit_script';
$SIG{'TERM'} = 'exit_script';

my $port = "/dev/ttyS0";
my $baud = 19200;
my $init_string = "z";		# Modem init string (excluding 'AT')
my $retry_on_busy = 3;		# Retry attempts on busy.
my $dial_timeout = 45;		# Number of seconds to wait for an answer.
my $wait_between_redial = 60;	# Number of seconds to wait between redial
				# attempts after busy.
my $logfile;

my $ok;
my $answer;
my $debug = 0;	# Set to 1 for debug output

GetOptions('port=s' => \$port);
my @split_path = File::Spec->splitpath($port);
my $portname = $split_path[2];
my $lockfile = "/var/lock/LCK..${portname}";

sub usage()
{
	print "Usage: notify_dial.pl [--port /path/to/tty] number [\"debug\" [logfile]]\n";
}

sub exit_script()
{
	unlink($lockfile);
	system ("sudo /etc/init.d/smsd start");
	exit;
}

sub printdebug
{
	my ($a) = @_;
	print($a);
    if ( $logfile )
	{
		open(LOGFILE, '>>'.$logfile);
		print(LOGFILE $a);
		close(LOGFILE);
	} else {
		print("Logfile closed\n");
	}
}


############### main() #################
my $number = $ARGV[0];
if ( ! $number ) {
	usage();
	exit;
}

if ( $ARGV[1] && $ARGV[1] eq 'debug' ) { $debug = 1; }
$logfile = $ARGV[2];

# Stop smsd
system ("sudo /etc/init.d/smsd stop");

# Check if lock file exist (serial port already in use)
if ( -e $lockfile) {
	if ($debug) { printdebug "ERROR: Serialport already in use!\n"; }
	exit_script();
}

# Create lockfile
open (LOCK,">$lockfile");
close (LOCK);

if ($debug) { printdebug "Your serial port is '$port'\n"; }

my $modem = new Device::Modem( port => $port, log => 'file,/opt/monitor/var/notify_dial.log' );

if( $modem->connect( baudrate => $baud, init_string => $init_string ) ) {
	if ($debug) { printdebug "Modem successfully initialized at $baud bps.\n"; }
} else {
	if ($debug) { printdebug "Cannot connect to $port serial port!: $!"; }
	exit_script();
}

# Set dialup timeout value.
$modem->send_init_string( 's7=' . $dial_timeout );

for ( my $attempt = 1; $attempt <= $retry_on_busy; $attempt++ ) {
	if ($debug) { printdebug('- trying to dial ['. $number. ']'. "\n"); }
	$modem->atsend( 'ATDT' . $number . ';' . Device::Modem::CR );
	$answer = $modem->answer( qr/[A-Z]/, 180 * 1000 );
	if (!$answer) { $answer='No answer'; }
	if ($debug) { printdebug "Modem answer: $answer\n"; }
	if ( $answer ne "BUSY" && $answer ne "No answer" ) {
		last;
	}
	if ( $attempt < $retry_on_busy ) {
		if ($debug) { printdebug "Waiting $wait_between_redial seconds to redial.\n"; }
		sleep($wait_between_redial);
	}
}

if ( $answer eq "BUSY" ) {
	if ($debug) { printdebug "BUSY, BUSY, BUSY -- We give up!\n"; }
} elsif ( $answer eq "No answer" ) {
	if ($debug) { printdebug "No answer from modem -- We give up!\n"; }
} else {
	if ($debug) { printdebug ("We have successfully dialed our number!\n"); }
}

# Clean up and exit
exit_script();

